import logging
import mock
import os
import shutil
import tempfile
import unittest

from cloudfoundry import tactics

# Set the log level high here to avoid messages from deployer in the test
# output. They are an artifact of partial specification for testing.
logging.basicConfig(level=logging.CRITICAL)


class TestGenerate(unittest.TestCase):
    def test_generate(self):
        d = tempfile.mkdtemp()
        g = tactics.GenerateTactic()
        g._run(None, cf_release=173, repo=d)
        self.assertTrue(
            os.path.exists(os.path.join(d, '173')))
        self.assertTrue(
            os.path.exists(os.path.join(d, '173', 'trusty')))
        shutil.rmtree(d)


class TestUpgradeCharm(unittest.TestCase):
    @mock.patch('shutil.make_archive')
    @mock.patch('jujuclient.Environment')
    def test_upgrade(self, menv, marchive):
        t = tactics.UpdateCharmTactic()
        # Use a qualified name here so we can run
        # w/o network access
        t._run(menv, charm_url='local:trusty/dea-v1-1',
               repo='build',
               cf_release=173)
        menv.add_local_charm.assert_called_once()
        marchive.assert_called_once()


class TestDeploy(unittest.TestCase):
    @mock.patch('deployer.charm.Charm.from_service')
    @mock.patch('jujuclient.Environment')
    def test_deploy(self, menv, mcharm):
        mcharm.charm_url.return_value = 'local:trusty/dea-v1'

        t = tactics.DeployTactic()
        t._run(menv, service=dict(service_name='local:trusty/dea-v1'),
               repo='build')
        menv.deploy.assert_called_once_with(
            'local:trusty/dea-v1', mock.ANY,
            config=None, num_units=1, constraints=None)


class TestRemoveService(unittest.TestCase):
    @mock.patch('jujuclient.Environment')
    def test_remove_service(self, menv):
        t = tactics.RemoveServiceTactic()
        t._run(menv, service_name="foo")
        menv.destroy_service.assert_called_once_with('foo')


class TestAddRelation(unittest.TestCase):
    @mock.patch('jujuclient.Environment')
    def test_add_relation(self, menv):
        t = tactics.AddRelationTactic()
        t._run(menv, endpoint_a="foo", endpoint_b="bar")
        menv.add_relation.assert_called_once_with('foo', 'bar')


class TestRemoveRelation(unittest.TestCase):
    @mock.patch('jujuclient.Environment')
    def test_remove_relation(self, menv):
        t = tactics.RemoveRelationTactic()
        t._run(menv, endpoint_a="foo", endpoint_b="bar")
        menv.remove_relation.assert_called_once_with('foo', 'bar')
